// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/matrix_sdk.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/util/json.dart';
import 'package:meta/meta.dart';

abstract class Event extends Identifiable<EventId> implements JsonWritable {
  @protected
  String get matrixType;

  @override
  final EventId id;

  final SentState sentState = SentState.sent;

  final UnsignedData unsigned;

  EventContent get content;

  Event(this.id, this.unsigned);

  factory Event.fromContent(EventContent content, RoomEventArgs args) {
    if (content is TextMessage) {
      return TextMessageEvent(content, args);
    } else {
      return null;
    }
  }

  @override
  Map<String, dynamic> toJson() => {
    'event_id': id.toJson(),
    'type': matrixType,
    'content': content?.toJson() ?? {},
    'unsigned': unsigned?.toJson()
  };

  @override
  String toString() => toJson().toString();
}

enum SentState {
  unsent,
  failed,
  sent
}

abstract class EventContent {

  String get matrixType;

  Map<String, dynamic> toJson() { return {}; }
}

class UnsignedData {
  final String transactionId;

  UnsignedData({@required this.transactionId});

  dynamic toJson() => {
    'transaction_id': transactionId
  };
}